package com.aitrainer.api.controller.diet

import com.aitrainer.api.model.diet.DietUserConsumption
import com.aitrainer.api.repository.diet.DietUserConsumptionRepository
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api")
class DietUserConsumptionController(private val dietUserConsumptionRepository: DietUserConsumptionRepository) {

    @PostMapping("/diet_user_consumption")
    fun insert(@RequestBody dietUserConsumption: DietUserConsumption): ResponseEntity<*> {
        return ResponseEntity.ok().body(dietUserConsumptionRepository.save(dietUserConsumption))
    }

    @PostMapping("/diet_user_consumption/{id}")
    fun update(@PathVariable(value = "id") id: Long, @RequestBody dietUserConsumption: DietUserConsumption): ResponseEntity<*> {
        return dietUserConsumptionRepository.findById(id).map { existingConsumption ->
            val updatedConsumption: DietUserConsumption = existingConsumption.copy(
                rawMaterialId = dietUserConsumption.rawMaterialId,
                dateConsumption = dietUserConsumption.dateConsumption,
                name = dietUserConsumption.name,
                quantity = dietUserConsumption.quantity,
                quantityUnit = dietUserConsumption.quantityUnit,
                cal = dietUserConsumption.cal,
                protein = dietUserConsumption.protein,
                fat = dietUserConsumption.fat,
                ch = dietUserConsumption.ch,
                sugar = dietUserConsumption.sugar
            )
            ResponseEntity.ok().body(dietUserConsumptionRepository.save(updatedConsumption))
        }.orElse(ResponseEntity.notFound().build())
    }

    @GetMapping("/diet_user_consumption/{dietUserId}")
    fun getByDietUserId(@PathVariable dietUserId: Long): ResponseEntity<List<DietUserConsumption>> {
        val list = dietUserConsumptionRepository.findByDietUserId(dietUserId)
        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }
}