package com.aitrainer.api.test

import com.aitrainer.api.model.CustomerConversation
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.content
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.status

@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerConversationTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }
    @Test
    fun `insert customer conversation`() {
        val customerConversation = CustomerConversation(1, 123, "2022-01-01 10:10:10", "What's up?", "Not much.")

        val gson = Gson()

        mockMvc.perform(
            post("/api/customer_conversation")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(gson.toJson(customerConversation))
            )
            .andExpect(status().isOk)
            .andExpect(content().json("""
                {
                    "id": 1,
                    "customerId": 123,
                    "conversationDate": "2022-01-01 10:10:10",
                    "question": "What's up?",
                    "answer": "Not much."
                }
            """.trimIndent()))
    }

    @Test
    fun `get customer conversation by customer id`() {
        mockMvc.perform(get("/api/customer_conversation/123")
            .header("Authorization", "Bearer $authToken"))
            .andExpect(status().isOk)
            .andExpect(content().json("""
                [
                    {
                        "id": 1,
                        "customerId": 123,
                        "conversationDate": "2022-01-01 10:10:10",
                        "question": "What's up?",
                        "answer": "Not much."
                    }
                ]
            """.trimIndent()))
    }
}
