package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.RawMaterial
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class RawMaterialTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private val rawMaterial = RawMaterial(

        name = "Tükörtojás",
        kcalMin = 70,
        kcalMax = 110,
        proteinMin = 19,
        proteinMax = 25,
        fatMin = 12,
        fatMax = 15,
        chMin = 1,
        chMax = 3,
        sugar = 0,
        storeId = 4,
    )
    private val rawMaterial2 = RawMaterial(

        name = "Tigris buci",
        kcalMin = 123,
        kcalMax = 140,
        proteinMin = 19,
        proteinMax = 25,
        fatMin = 30,
        fatMax = 34,
        chMin = 50,
        chMax = 60,
        sugar = 5,
        storeId = 2,
    )
    private val rawMaterial3 = RawMaterial(

        name = "Tükörtojás",
        kcalMin = 75,
        kcalMax = 112,
        proteinMin = 12,
        proteinMax = 20,
        fatMin = 12,
        fatMax = 15,
        chMin = 1,
        chMax = 3,
        sugar = 0,
        storeId = 2,
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert rawMaterials successfully`() {

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/raw_material")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(rawMaterial))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.sugar").value(0))
        .andExpect(jsonPath("$.storeId").value(4))
        .andExpect(jsonPath("$.proteinMax").value(25))

        val mvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/raw_material")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(rawMaterial2))
        )
            .andExpect(status().isOk)
            .andReturn()

        val gson= Gson()
        val newRawMaterialJson = mvcResult.response.contentAsString
        val newRawMaterial = gson.fromJson(newRawMaterialJson, RawMaterial::class.java)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/raw_material")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(rawMaterial3))
        )
            .andExpect(status().isOk)

         // Act & Assert
        mockMvc.perform(get("/api/raw_material" )
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Tükörtojás"))
            .andExpect(jsonPath("$.[1].name").value("Tigris buci"))
            .andExpect(jsonPath("$.[2].storeId").value(2))
            .andExpect(jsonPath("$.[2].name").value("Tükörtojás"))


        // Act & Assert
        mockMvc.perform(get("/api/raw_material/${newRawMaterial.id}")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.name").value("Tigris buci"))

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
